import com.fasterxml.jackson.annotation.JsonValue

data class Money(@JsonValue private val cents: Int) {
    operator fun plus(other: Money): Money {
        return Money(this.cents + other.cents)
    }

    operator fun minus(other: Money): Money {
        return Money(this.cents - other.cents)
    }

    companion object {
        fun fromDollarsAndCents(dollars: Int, cents: Int): Money {
            require((cents <= 99) and (cents >= 0))
            return if (dollars >= 0) {
                Money(dollars * 100 + cents)
            } else Money(dollars * 100 - cents)
        }

        fun fromString(s: String): Money {
            val splitString = s.split('.')
            require(splitString.size == 2)

            val cents = splitString.get(1).toInt()

            val dollarSplit = splitString.get(0).split('$')
            return if (dollarSplit.get(0) == "-") {
                fromDollarsAndCents(dollars = dollarSplit.get(1).toInt().unaryMinus(),
                        cents = cents)
            } else {
                fromDollarsAndCents(dollars = dollarSplit.get(1).toInt(),
                        cents = cents)
            }
        }
    }
}